import { scaleLinear, axisLeft, extent, line } from 'd3';
import {
  xAxis,
  yAxis,
  xAxisLabel,
  yAxisLabel,
} from '@curran/axis-labels';

export const viz = (
  svg,
  {
    data,
    xValue,
    xAxisLabelText,
    xAxisLabelOffset,
    yValue,
    yTwoValue,
    yAxisLabelText,
    yAxisLabelOffset,
    marginLeft,
    marginTop,
    marginRight,
    marginBottom,
    width,
    height,
    innerRectFill,
    circleRadius,
    circleOpacity,
  },
) => {
  const xScale = scaleLinear(extent(data, xValue), [
    marginLeft,
    width - marginRight,
  ]);
  const yScale = scaleLinear(extent(data, yValue), [
    height - marginBottom,
    marginTop,
  ]);

  const yScaleTwo = scaleLinear(extent(data, yTwoValue), [
    height - marginBottom,
    marginTop,
  ]);

  const innerWidth = width - marginLeft - marginRight;
  const innerHeight = height - marginTop - marginBottom;

  svg
    .selectAll('rect')
    .data([null])
    .join('rect')
    .attr('x', marginLeft)
    .attr('y', marginTop)
    .attr('width', innerWidth)
    .attr('height', innerHeight)
    .attr('fill', innerRectFill);

  // Axes
  xAxis(svg, { height, marginBottom, xScale });
  yAxis(svg, { marginLeft, yScale });

  // Axis Labels
  xAxisLabel(svg, {
    xAxisLabelText,
    width,
    height,
    marginBottom,
    xAxisLabelOffset,
  });
  yAxisLabel(svg, {
    yAxisLabelText,
    yAxisLabelOffset,
    height,
  });

  const lineGenerator = line(
    (d) => xScale(xValue(d)),
    (d) => yScale(yValue(d)),
  ).defined(yValue);

  const linePopulation = line(
    (d) => xScale(xValue(d)),
    (d) => yScaleTwo(yTwoValue(d)),
  ).defined(yTwoValue);

  // A line
  svg
    .selectAll('path.mark')
    .data([null])
    .join('path')
    .attr('class', 'mark')
    .attr('fill', 'none')
    .attr('stroke', 'black')
    .attr('d', lineGenerator(data));

  svg
    .selectAll('path.population')
    .data([null])
    .join('path')
    .attr('class', 'population')
    .attr('fill', 'none')
    .attr('stroke', 'red')
    .attr('d', linePopulation(data));
};
